// $Id: test-env-2.C,v 1.2 2002/10/20 13:09:42 dvermeir Exp $
#include <iostream>
#include <sstream>
#include "env.h"

// Test Env::substitute

using namespace std;
using namespace Dv;
using namespace Util;

string 
substitute(const Env& e, const string& s) {
cout << "input = " << s << endl;
istringstream iss(s);
ostringstream oss;
e.substitute(oss, iss);
string out(oss.str());
cerr << "output = " << out << endl;
return out;
}

int
main(int, char**) {

static const string output("abc def ghi");
try {
  {
  // simple ${xyz}
  Env e;

  e["xyz"] = "def";
  string out(substitute(e, "abc ${xyz} ghi"));
  if (out != output) 
    throw runtime_error(out + ": but expected \"" + output + "\"");
  }

  {
  // more complicated: ${0::xyz}
  Env e;
  e[0]["xyz"] = "def";

  string out(substitute(e, "abc ${0::xyz} ghi"));
  if (out != output) 
    throw runtime_error(out + ": but expected \"" + output + "\"");

  }

  {
  // nested: ${0::xyz}
  Env e;
  e["xyz"] = "def";
  e["abc"] = "${xyz}";

  string out(substitute(e, "abc ${abc} ghi"));
  if (out != output) 
    throw runtime_error(out + ": but expected \"" + output + "\"");

  }

  {
  // array: ${*}{${x} }
  Env e;
  e[0]["x"] = "abc";
  e[1]["x"] = "def";

  string out(substitute(e, "${*}{${x} }ghi"));
  if (out != output) 
    throw runtime_error(out + ": but expected \"" + output + "\"");

  }

  {
  // array: ${fred::*}{${x} }
  Env e;
  e["fred"][0]["x"] = "abc";
  e["fred"][1]["x"] = "def";

  string out(substitute(e, "${fred::*}{${x} }ghi"));
  if (out != output) 
    throw runtime_error(out + ": but expected \"" + output + "\"");

  }

  {
  // array: ${fred::*}{${y} }
  Env e;
  e["fred"][0]["y"] = "${x}";
  e["fred"][0]["x"] = "abc";
  e["fred"][1]["x"] = "def";
  e["fred"][1]["y"] = "${x}";

  string out(substitute(e, "${fred::*}{${y} }ghi"));
  if (out != output) 
    throw runtime_error(out + ": but expected \"" + output + "\"");

  }

  }
catch (exception& e) {
  cerr << e.what() << endl;
  return 1;
  }
return 0;
}
